
require "rest-client"
require "tempfile"
require "zipruby"
require "json"
require "base64"

class Mobiorm

    MOBIORM_VERSION = "1.0"
    HTTP_SERVICE = "https://api-mobile-orm.vertabelo.com/" + MOBIORM_VERSION + "/generate/"
    VERTABELO_API_XML = "https://my.vertabelo.com/api/xml/"
        
    def self.downloadModel(token, id, tag)
        RestClient::Resource.new(VERTABELO_API_XML + id + (tag ? "/" + tag : ""), token, "") { 
            |response, request, result, &block|
            case response.code
            when 200
                response
            else
                abort "Could not download you model, HTTP request ended with code: " + response.code.to_s + "."
            end
        }
    end

    def self.generateOrm(file, platform, classPrefix, propertyPrefix, package)
        parameters = Hash.new
        parameters['vertabeloXML'] = file

        if platform == "objectivec"
            if propertyPrefix
                parameters['propertyPrefix'] = propertyPrefix
            end

            if classPrefix
                parameters['classPrefix'] = classPrefix
            end
        elsif platform == "android"
            if package
                parameters['package'] = package
            end
        end        

        RestClient.post(HTTP_SERVICE + platform, parameters) { 
            |response, request, result, &block|
            case response.code
            when 200
                response
            else
                abort "Generation unsuccessful, HTTP request ended with code: " + response.code.to_s + "."
            end
        }
    end

    def self.saveOrm(response, path)
        Zip::Archive.open_buffer(Base64.decode64(response)) do |archive|
            archive.each do |entry|
                if not entry.directory?
                    open(path + File.basename(entry.name), "wb") do |f|
                        f << entry.read
                    end
                end
            end
        end
    end

    def self.getOrm(file, path, platform, classPrefix, propertyPrefix, package)
        response = JSON.parse(generateOrm(file, platform, classPrefix, propertyPrefix, package))
        if response["status"] == "OK"
            response["result"]["errors"].each do |error|
                puts "ERROR: " + error
            end
            response["result"]["warnings"].each do |warning|
                puts "WARNING: " + warning
            end
            self.saveOrm(response["result"]["content"], path)
            puts
            puts "Generation successful, files saved in: " + path
            puts
        elsif response["status"] == "ERROR"
            abort "Error during ORM generation: " + response["result"]["error"]
        else
            abort "Error during ORM generation"
        end
    end

    def self.getOrmFromVertabelo(dest, platform, token, id, tag, classPrefix, propertyPrefix, package)
        path = "" + dest + "/"
        resource = self.downloadModel(token, id, tag)
        Tempfile.open("temp.xml") do |f|
            f.write(resource.get)
            f.rewind
            self.getOrm(f, path, platform, classPrefix, propertyPrefix, package)
        end
    end

    def self.getOrmFromFile(dest, platform, xml, classPrefix, propertyPrefix, package)
        path = "" + dest + "/"
        File.open(xml) do |f|
            self.getOrm(f, path, platform, classPrefix, propertyPrefix, package)
        end
    end
end
